@@ -88,15 +88,14 @@ panel_applet_toggle_locked (AppletInfo *info)
 }
 
 static void
-panel_applet_lock (GtkWidget  *menuitem,
-		   AppletInfo *info)
+panel_applet_lock (GtkCheckMenuItem *menuitem,
+		   AppletInfo       *info)
 {
 	gboolean locked;
 
 	locked = panel_applet_toggle_locked (info);
 
-	gtk_label_set_label (GTK_LABEL (GTK_BIN (menuitem)->child),
-			     locked ? _("Un_lock") : _("_Lock"));
+	gtk_check_menu_item_set_active (menuitem, locked);
 
 	if (info->move_item)
 		gtk_widget_set_sensitive (info->move_item, !locked);
@@ -157,6 +156,46 @@ panel_applet_recreate_menu (AppletInfo	*info)
 	info->menu = panel_applet_create_menu (info);
 }
 
+static void
+panel_applet_locked_change_notify (GConfClient *client,
+				   guint        cnxn_id,
+				   GConfEntry  *entry,
+				   GtkWidget   *applet)
+{
+	GConfValue  *value;
+	gboolean     locked;
+	gboolean     applet_locked;
+	AppletInfo  *info;
+	PanelWidget *panel_widget;
+
+	g_assert (applet != NULL);
+
+	info = (AppletInfo  *) g_object_get_data (G_OBJECT (applet),
+						  "applet_info");
+	if (info == NULL)
+		return;
+
+	value = gconf_entry_get_value (entry);
+	if (value->type != GCONF_VALUE_BOOL)
+		return;
+
+	locked = gconf_value_get_bool (value);
+
+	panel_widget = PANEL_WIDGET (info->widget->parent);
+	applet_locked = panel_widget_get_applet_locked (panel_widget,
+							info->widget);
+
+	if ((locked && applet_locked) || !(locked || applet_locked))
+		return;
+
+	panel_applet_toggle_locked (info);
+
+	if (info->type == PANEL_OBJECT_BONOBO)
+		panel_applet_frame_sync_menu_state (PANEL_APPLET_FRAME (info->widget));
+	else
+		panel_applet_recreate_menu (info);
+}
+
 static void
 applet_remove_callback (GtkWidget  *widget,
 			AppletInfo *info)
@@ -474,18 +513,9 @@ panel_applet_create_menu (AppletInfo *info)
 				  G_CALLBACK (applet_remove_callback), info);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 		gtk_widget_set_sensitive (menuitem, removable);
-
-		menuitem = gtk_image_menu_item_new ();
-		image = gtk_image_new ();
-		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, image, locked ? _("Un_lock") : _("_Lock"), FALSE);
-		g_signal_connect (menuitem, "activate",
-				  G_CALLBACK (panel_applet_lock), info);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-		gtk_widget_set_sensitive (menuitem, lockable);
 		
 		menuitem = gtk_image_menu_item_new ();
-		image = gtk_image_new ();
-		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, image, _("_Move"), FALSE);
+		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, NULL, _("_Move"), FALSE);
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (move_applet_callback), info);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
@@ -497,6 +527,19 @@ panel_applet_create_menu (AppletInfo *info)
 		g_object_add_weak_pointer (G_OBJECT (menuitem),
 					   (gpointer *) &info->move_item);
 
+		menuitem = gtk_separator_menu_item_new ();
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+		gtk_widget_show (menuitem);
+
+		menuitem = gtk_check_menu_item_new ();
+		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, NULL, _("_Lock To Panel"), FALSE);
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem),
+						locked);
+		g_signal_connect (menuitem, "toggled",
+				  G_CALLBACK (panel_applet_lock), info);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+		gtk_widget_set_sensitive (menuitem, lockable);
+
 		added_anything = TRUE;
 	}
 
@@ -542,6 +585,8 @@ panel_applet_position_menu (GtkMenu   *menu,
 	GdkScreen      *screen;
 	int             menu_x = 0;
 	int             menu_y = 0;
+	int             pointer_x;
+	int             pointer_y;
 
 	g_return_if_fail (PANEL_IS_WIDGET (applet->parent));
 
@@ -550,6 +595,7 @@ panel_applet_position_menu (GtkMenu   *menu,
 	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
 
 	gdk_window_get_origin (applet->window, &menu_x, &menu_y);
+	gtk_widget_get_pointer (applet, &pointer_x, &pointer_y);
 
 	if (GTK_WIDGET_NO_WINDOW (applet)) {
 		menu_x += applet->allocation.x;
@@ -557,11 +603,17 @@ panel_applet_position_menu (GtkMenu   *menu,
 	}
 
 	if (PANEL_WIDGET (applet->parent)->orient == GTK_ORIENTATION_HORIZONTAL) {
+		if (requisition.width < pointer_x)
+			menu_x += MIN (pointer_x, applet->allocation.width - requisition.width);
+
 		if (menu_y > gdk_screen_get_height (screen) / 2)
 			menu_y -= requisition.height;
 		else
 			menu_y += applet->allocation.height;
 	} else {
+		if (requisition.height < pointer_y)
+			menu_y += MIN (pointer_y, applet->allocation.height - requisition.height);
+
 		if (menu_x > gdk_screen_get_width (screen) / 2)
 			menu_x -= requisition.width;
 		else
@@ -677,8 +729,9 @@ panel_applet_destroy (GtkWidget  *widget,
 		}
 	}
 
-	panel_lockdown_notify_remove (G_CALLBACK (panel_applet_recreate_menu),
-				      info);
+	if (info->type != PANEL_OBJECT_BONOBO)
+		panel_lockdown_notify_remove (G_CALLBACK (panel_applet_recreate_menu),
+					      info);
 
 	if (info->menu)
 		g_object_unref (info->menu);
@@ -1062,15 +1115,20 @@ panel_applet_list_applets (void)
 }
 
 AppletInfo *
-panel_applet_get_by_type (PanelObjectType object_type)
+panel_applet_get_by_type (PanelObjectType object_type, GdkScreen *screen)
 {
 	GSList *l;
 
 	for (l = registered_applets; l; l = l->next) {
 		AppletInfo *info = l->data;
 
-		if (info->type == object_type)
-			return info;
+		if (info->type == object_type) {
+			if (screen) {
+				if (screen == gtk_widget_get_screen (info->widget))
+					return info;
+			} else
+				return info;
+		}
 	}
 
 	return NULL;
@@ -1088,6 +1146,7 @@ panel_applet_register (GtkWidget       *applet,
 		       const char      *id)
 {
 	AppletInfo *info;
+	const char *key;
 	
 	g_return_val_if_fail (applet != NULL && panel != NULL, NULL);
 
@@ -1109,8 +1168,16 @@ panel_applet_register (GtkWidget       *applet,
 
 	g_object_set_data (G_OBJECT (applet), "applet_info", info);
 
-	panel_lockdown_notify_add (G_CALLBACK (panel_applet_recreate_menu),
-				   info);
+	if (type != PANEL_OBJECT_BONOBO)
+		panel_lockdown_notify_add (G_CALLBACK (panel_applet_recreate_menu),
+					   info);
+
+	key = panel_gconf_full_key ((type == PANEL_OBJECT_BONOBO) ?
+				     PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS,
+				    id, "locked");
+	panel_gconf_notify_add_while_alive (key,
+					    (GConfClientNotifyFunc) panel_applet_locked_change_notify,
+					    G_OBJECT (applet));
 
 	if (type == PANEL_OBJECT_DRAWER) {
 		Drawer *drawer = data;
