@@ -106,7 +106,7 @@ static void panel_applet_frame_activated (Bonobo_Unknown     object,
 					  CORBA_Environment *ev,
 					  gpointer          *data);
 
-static void
+void
 panel_applet_frame_sync_menu_state (PanelAppletFrame *frame)
 {
 	PanelWidget *panel_widget;
@@ -125,9 +125,9 @@ panel_applet_frame_sync_menu_state (PanelAppletFrame *frame)
 	locked = panel_widget_get_applet_locked (panel_widget, GTK_WIDGET (frame));
 
 	bonobo_ui_component_set_prop (frame->priv->ui_component,
-				      "/popups/button3/placeholder/lock",
-				      "label",
-				      locked ? _("Un_lock") : _("_Lock"),
+				      "/commands/LockAppletToPanel",
+				      "state",
+				      locked ? "1" : "0",
 				      NULL);
 
 	/* First sensitivity */
@@ -254,10 +254,32 @@ popup_handle_remove (BonoboUIComponent *uic,
 }
 
 static void
-popup_handle_lock (BonoboUIComponent *uic,
-		   PanelAppletFrame  *frame,
-		   const char        *verb)
+listener_popup_handle_lock (BonoboUIComponent            *uic,
+			    const char                   *path,
+			    Bonobo_UIComponent_EventType  type,
+			    const char                   *state,
+			    gpointer                      data)
 {
+	PanelAppletFrame *frame;
+	AppletInfo       *info;
+	gboolean          s;
+
+	g_assert (!strcmp (path, "LockAppletToPanel"));
+
+	if (type != Bonobo_UIComponent_STATE_CHANGED)
+		return;
+
+	if (!state)
+		return;
+
+	frame = (PanelAppletFrame *) data;
+	info = frame->priv->applet_info;
+	s = (strcmp (state, "1") == 0);
+
+	if (panel_widget_get_applet_locked (PANEL_WIDGET (info->widget->parent),
+					    info->widget) == s)
+		return;
+
 	panel_applet_toggle_locked (frame->priv->applet_info);
 
 	panel_applet_frame_sync_menu_state (frame);
@@ -282,13 +304,11 @@ popup_handle_move (BonoboUIComponent *uic,
 
 static BonoboUIVerb popup_verbs [] = {
         BONOBO_UI_UNSAFE_VERB ("RemoveAppletFromPanel", popup_handle_remove),
-	BONOBO_UI_UNSAFE_VERB ("LockAppletToPanel",     popup_handle_lock),
         BONOBO_UI_UNSAFE_VERB ("MoveApplet",            popup_handle_move),
 
         BONOBO_UI_VERB_END
 };
 
-
 static void
 panel_applet_frame_load (const gchar *iid,
 			 PanelWidget *panel,
@@ -446,7 +466,9 @@ panel_applet_frame_get_background_string (PanelAppletFrame    *frame,
 		switch (frame->priv->orientation) {
 		case PANEL_ORIENTATION_TOP:
 		case PANEL_ORIENTATION_BOTTOM:
-			x += frame->priv->handle_rect.width;
+			if (gtk_widget_get_direction (GTK_WIDGET (frame)) !=
+			    GTK_TEXT_DIR_RTL)
+				x += frame->priv->handle_rect.width;
 			break;
 		case PANEL_ORIENTATION_LEFT:
 		case PANEL_ORIENTATION_RIGHT:
@@ -673,7 +695,15 @@ panel_applet_frame_size_allocate (GtkWidget     *widget,
 		frame->priv->handle_rect.width  = HANDLE_SIZE;
 		frame->priv->handle_rect.height = allocation->height;
 
-		new_allocation.x      = HANDLE_SIZE;
+		if (gtk_widget_get_direction (GTK_WIDGET (frame)) !=
+		    GTK_TEXT_DIR_RTL) {
+			frame->priv->handle_rect.x = 0;
+			new_allocation.x = HANDLE_SIZE;
+		} else {
+			frame->priv->handle_rect.x = allocation->width - HANDLE_SIZE;
+			new_allocation.x = 0;
+		}
+
 		new_allocation.y      = 0;
 		new_allocation.width  = allocation->width - HANDLE_SIZE;
 		new_allocation.height = allocation->height;
@@ -1235,9 +1265,9 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 	   exception can happen, so handle it */
 	if (BONOBO_EX (ev) || object == CORBA_OBJECT_NIL) {
 		error = bonobo_exception_get_text (ev);
-		panel_applet_frame_loading_failed (frame, frame_act->id);
 		g_warning (G_STRLOC ": failed to load applet %s:\n%s",
 			   frame->priv->iid, error);
+		panel_applet_frame_loading_failed (frame, frame_act->id);
 		g_free (frame_act->id);
 		g_free (frame_act);
 		g_free (error);
@@ -1255,9 +1285,9 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 	bonobo_object_release_unref (object, NULL);
 
 	if (!widget) {
-		panel_applet_frame_loading_failed (frame, frame_act->id);
 		g_warning (G_STRLOC ": failed to load applet %s",
 			   frame->priv->iid);
+		panel_applet_frame_loading_failed (frame, frame_act->id);
 		g_free (frame_act->id);
 		g_free (frame_act);
 		return;
@@ -1265,9 +1295,9 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 
 	control_frame = bonobo_widget_get_control_frame (BONOBO_WIDGET (widget));
 	if (control_frame == NULL) {
-		panel_applet_frame_loading_failed (frame, frame_act->id);
 		g_warning (G_STRLOC ": failed to load applet %s "
 			   "(can't get control frame)", frame->priv->iid);
+		panel_applet_frame_loading_failed (frame, frame_act->id);
 		gtk_object_sink (GTK_OBJECT (widget));
 		g_free (frame_act->id);
 		g_free (frame_act);
@@ -1279,11 +1309,11 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 							       &corba_ev);
 	if (frame->priv->property_bag == NULL || BONOBO_EX (&corba_ev)) {
 		error = bonobo_exception_get_text (&corba_ev);
-		panel_applet_frame_loading_failed (frame, frame_act->id);
 		CORBA_exception_free (&corba_ev);
 		g_warning (G_STRLOC ": failed to load applet %s "
 			   "(can't get property bag):\n%s",
 			   frame->priv->iid, error);
+		panel_applet_frame_loading_failed (frame, frame_act->id);
 		gtk_object_sink (GTK_OBJECT (widget));
 		g_free (frame_act->id);
 		g_free (frame_act);
@@ -1302,11 +1332,11 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 							  &corba_ev);
 	if (frame->priv->ui_component == NULL || BONOBO_EX (&corba_ev)) {
 		error = bonobo_exception_get_text (&corba_ev);
-		panel_applet_frame_loading_failed (frame, frame_act->id);
 		CORBA_exception_free (&corba_ev);
 		g_warning (G_STRLOC ": failed to load applet %s "
 			   "(can't get popup component):\n%s",
 			   frame->priv->iid, error);
+		panel_applet_frame_loading_failed (frame, frame_act->id);
 		gtk_object_sink (GTK_OBJECT (widget));
 		g_free (frame_act->id);
 		g_free (frame_act);
@@ -1317,15 +1347,20 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 	bonobo_ui_util_set_ui (frame->priv->ui_component, DATADIR,
 			       "GNOME_Panel_Popup.xml", "panel", NULL);
 
+	bonobo_ui_component_add_listener (frame->priv->ui_component,
+					  "LockAppletToPanel",
+					  listener_popup_handle_lock,
+					  frame);
+
 	bonobo_ui_component_add_verb_list_with_data (
 		frame->priv->ui_component, popup_verbs, frame);
 
 	control = bonobo_control_frame_get_control (control_frame);
 	if (!control) {
 		CORBA_exception_free (&corba_ev);
-		panel_applet_frame_loading_failed (frame, frame_act->id);
 		g_warning (G_STRLOC ": failed to load applet %s "
 			   "(can't get control)", frame->priv->iid);
+		panel_applet_frame_loading_failed (frame, frame_act->id);
 		gtk_object_sink (GTK_OBJECT (widget));
 		g_free (frame_act->id);
 		g_free (frame_act);
@@ -1335,9 +1370,9 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 	frame->priv->applet_shell = panel_applet_frame_get_applet_shell (control);
 	if (frame->priv->applet_shell == CORBA_OBJECT_NIL) {
 		CORBA_exception_free (&corba_ev);
-		panel_applet_frame_loading_failed (frame, frame_act->id);
 		g_warning (G_STRLOC ": failed to load applet %s "
 			   "(can't get applet shell)", frame->priv->iid);
+		panel_applet_frame_loading_failed (frame, frame_act->id);
 		gtk_object_sink (GTK_OBJECT (widget));
 		g_free (frame_act->id);
 		g_free (frame_act);
