@@ -45,6 +45,7 @@
 #include <libgnomeui/gnome-ui-init.h>
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
+#include <X11/Xatom.h>
 
 #include "panel-applet.h"
 #include "panel-applet-private.h"
@@ -75,8 +76,6 @@ struct _PanelAppletPrivate {
         int                         size_hints_len;
 
 	gboolean                    moving_focus_out;
-	int                         focusable_child;
-	guint                       hierarchy_changed_id;
 
 	gboolean                    locked_down;
 };
@@ -107,7 +106,7 @@ enum {
 	PROPERTY_BACKGROUND_IDX,
 	PROPERTY_FLAGS_IDX,
 	PROPERTY_SIZE_HINTS_IDX,
-	PROPERTY_LOCKED_DOWN_IDX,
+	PROPERTY_LOCKED_DOWN_IDX
 };
 
 static void
@@ -326,6 +325,141 @@ panel_applet_get_locked_down (PanelApplet *applet)
 	return applet->priv->locked_down;
 }
 
+static Atom _net_wm_window_type = None;
+static Atom _net_wm_window_type_dock = None;
+static Atom _net_active_window = None;
+
+static void
+panel_applet_init_atoms (Display *xdisplay)
+{
+	if (_net_wm_window_type == None)
+		_net_wm_window_type = XInternAtom (xdisplay,
+						   "_NET_WM_WINDOW_TYPE",
+						   False);
+
+	if (_net_wm_window_type_dock == None)
+		_net_wm_window_type_dock = XInternAtom (xdisplay,
+							"_NET_WM_WINDOW_TYPE_DOCK",
+							False);
+
+	if (_net_active_window == None)
+		_net_active_window = XInternAtom (xdisplay,
+						  "_NET_ACTIVE_WINDOW",
+						  False);
+}
+
+static Window
+panel_applet_find_toplevel_dock_window (PanelApplet *applet,
+					Display	    *xdisplay)
+{
+	GtkWidget  *toplevel;
+	Window	    xwin;
+	Window	    root, parent, *child;
+	int	    num_children;
+
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (applet));
+	if (!GTK_WIDGET_REALIZED (toplevel))
+		return None;
+
+	xwin = GDK_WINDOW_XID (toplevel->window);
+
+	child = NULL;
+	parent = root = None;
+	do {
+		Atom	type_return;
+		Atom	window_type;
+		int	format_return;
+		gulong	number_return, bytes_after_return;
+		guchar *data_return;
+
+		XGetWindowProperty (xdisplay,
+				    xwin,
+				    _net_wm_window_type,
+				    0, 1, False,
+				    XA_ATOM,
+				    &type_return, &format_return,
+				    &number_return,
+				    &bytes_after_return,
+				    &data_return);
+
+		if (type_return == XA_ATOM) {
+			window_type = *(Atom *) data_return;
+
+			XFree (data_return);
+			data_return = NULL;
+
+			if (window_type == _net_wm_window_type_dock)
+				return xwin;
+		}
+
+		if (!XQueryTree (xdisplay,
+			   xwin,
+			   &root, &parent, &child,
+			   (guint *) &num_children)) {
+			   return None;
+		}
+
+		if (child && num_children > 0)
+			XFree (child);
+
+		xwin = parent;
+
+	} while (xwin != None && xwin != root);
+
+	return None;
+}
+
+/* This function
+ *   1) Gets the window id of the panel that contains the applet
+ *	using XQueryTree and XGetWindowProperty to find an ancestor
+ *	window with the _NET_WM_WINDOW_TYPE_DOCK window type.
+ *   2) Sends a _NET_ACTIVE_WINDOW message to get that panel focused
+ */
+void
+panel_applet_request_focus (PanelApplet	 *applet,
+			    guint32	  timestamp)
+{
+	GdkScreen  *screen;
+	GdkWindow  *root;
+	GdkDisplay *display;
+	Display	   *xdisplay;
+	Window	    dock_xwindow;
+	Window	    xroot;
+	XEvent	    xev;
+
+	g_return_if_fail (PANEL_IS_APPLET (applet));
+
+	screen	= gtk_widget_get_screen (GTK_WIDGET (applet));
+	root	= gdk_screen_get_root_window (screen);
+	display = gdk_screen_get_display (screen);
+
+	xdisplay = GDK_DISPLAY_XDISPLAY (display);
+	xroot	 = GDK_WINDOW_XWINDOW (root);
+
+	panel_applet_init_atoms (xdisplay);
+
+	dock_xwindow = panel_applet_find_toplevel_dock_window (applet, xdisplay);
+	if (dock_xwindow == None)
+		return;
+
+	xev.xclient.type	 = ClientMessage;
+	xev.xclient.serial	 = 0;
+	xev.xclient.send_event	 = True;
+	xev.xclient.window	 = dock_xwindow;
+	xev.xclient.message_type = _net_active_window;
+	xev.xclient.format	 = 32;
+	xev.xclient.data.l[0]	 = 1; /* requestor type; we're an app, I guess */
+	xev.xclient.data.l[1]	 = timestamp;
+	xev.xclient.data.l[2]	 = None; /* "currently active window", supposedly */
+	xev.xclient.data.l[3]	 = 0;
+	xev.xclient.data.l[4]	 = 0;
+
+	XSendEvent (xdisplay,
+		    xroot, False,
+		    SubstructureRedirectMask | SubstructureNotifyMask,
+		    &xev);
+}
+
 void
 panel_applet_setup_menu (PanelApplet        *applet,
 			 const gchar        *xml,
@@ -364,7 +498,8 @@ panel_applet_setup_menu_from_file (PanelApplet        *applet,
 		opt_datadir = PANEL_APPLET_DATADIR;
 
 	if (!opt_app_name)
-		opt_app_name = app_name = g_strdup_printf ("%d", getpid ());
+		opt_app_name = app_name = g_strdup_printf ("%lu",
+							   (unsigned long) getpid ());
 
 	popup_component = panel_applet_get_popup_component (applet);
 
@@ -420,25 +555,23 @@ panel_applet_finalize (GObject *object)
 	parent_class->finalize (object);
 }
 
-static GtkWidget*
-panel_applet_container_has_focusable_child (GtkWidget *widget)
+static gboolean
+container_has_focusable_child (GtkContainer *container)
 {
-	GtkContainer *container;
 	GtkWidget *child;
 	GList *list;
 	GList *t;
-	GtkWidget *retval = NULL;
+	gboolean retval = FALSE;
 
-	container = GTK_CONTAINER (widget);
 	list = gtk_container_get_children (container);
 
 	for (t = list; t; t = t->next) {
 		child = GTK_WIDGET (t->data);
 		if (GTK_WIDGET_CAN_FOCUS (child)) {
-			retval = child;
+			retval = TRUE;
 			break;
 		} else if (GTK_IS_CONTAINER (child)) {
-			retval = panel_applet_container_has_focusable_child (child);
+			retval = container_has_focusable_child (GTK_CONTAINER (child));
 			if (retval)
 				break;
 		}
@@ -447,32 +580,6 @@ panel_applet_container_has_focusable_child (GtkWidget *widget)
 	return retval;	
 }
 
-static void
-panel_applet_hierarchy_changed_cb (GtkWidget *widget)
-{
-	PanelApplet *applet = PANEL_APPLET (widget);
-
-	applet->priv->focusable_child = -1;
-}
-
-static gboolean
-panel_applet_has_focusable_child (PanelApplet *applet)
-{
-	if (applet->priv->focusable_child == -1) {
-		GtkWidget *focusable_child;
-
-		if (!applet->priv->hierarchy_changed_id)
-			applet->priv->hierarchy_changed_id = 
-				g_signal_connect (applet, "hierarchy-changed", 
-				  		  G_CALLBACK (panel_applet_hierarchy_changed_cb),
-				  		  NULL);
-
-		focusable_child = panel_applet_container_has_focusable_child (GTK_WIDGET (applet));
-		applet->priv->focusable_child = (focusable_child != NULL);
-	}
-	return  (applet->priv->focusable_child != 0);
-}
-
 static void
 panel_applet_position_menu (GtkMenu   *menu,
 			    int       *x,
@@ -485,6 +592,8 @@ panel_applet_position_menu (GtkMenu   *menu,
 	GdkScreen      *screen;
 	int             menu_x = 0;
 	int             menu_y = 0;
+	int             pointer_x;
+	int             pointer_y;
 
 	g_return_if_fail (PANEL_IS_APPLET (widget));
 
@@ -495,17 +604,24 @@ panel_applet_position_menu (GtkMenu   *menu,
 	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
 
 	gdk_window_get_origin (widget->window, &menu_x, &menu_y);
+	gtk_widget_get_pointer (widget, &pointer_x, &pointer_y);
 
 	menu_x += widget->allocation.x;
 	menu_y += widget->allocation.y;
 
 	if (applet->priv->orient == PANEL_APPLET_ORIENT_UP ||
 	    applet->priv->orient == PANEL_APPLET_ORIENT_DOWN) {
+		if (requisition.width < pointer_x)
+			menu_x += MIN (pointer_x, widget->allocation.width - requisition.width);
+
 		if (menu_y > gdk_screen_get_height (screen) / 2)
 			menu_y -= requisition.height;
 		else
 			menu_y += widget->allocation.height;
 	} else  {
+		if (requisition.height < pointer_y)
+			menu_y += MIN (pointer_y, widget->allocation.height - requisition.height);
+
 		if (menu_x > gdk_screen_get_width (screen) / 2)
 			menu_x -= requisition.width;
 		else
@@ -531,7 +647,7 @@ panel_applet_can_focus (GtkWidget *widget)
 	if (!PANEL_IS_APPLET (widget))
 		return FALSE;
 
-	return !panel_applet_has_focusable_child (PANEL_APPLET (widget));
+	return !container_has_focusable_child (GTK_CONTAINER (widget));
 }
 
 static gboolean
@@ -540,7 +656,7 @@ panel_applet_button_press (GtkWidget      *widget,
 {
 	PanelApplet *applet = PANEL_APPLET (widget);
 
-	if (!panel_applet_has_focusable_child (applet)) {
+	if (!container_has_focusable_child (GTK_CONTAINER (applet))) {
 		if (!GTK_WIDGET_HAS_FOCUS (widget)) {
 			GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_FOCUS);
 			gtk_widget_grab_focus (widget);
@@ -1332,10 +1448,10 @@ panel_applet_class_init (PanelAppletClass *klass)
                               G_STRUCT_OFFSET (PanelAppletClass, change_orient),
                               NULL,
 			      NULL,
-                              panel_applet_marshal_VOID__INT,
+                              panel_applet_marshal_VOID__UINT,
                               G_TYPE_NONE,
 			      1,
-			      G_TYPE_INT);
+			      G_TYPE_UINT);
 
 	panel_applet_signals [CHANGE_SIZE] =
                 g_signal_new ("change_size",
@@ -1356,11 +1472,11 @@ panel_applet_class_init (PanelAppletClass *klass)
                               G_STRUCT_OFFSET (PanelAppletClass, change_background),
                               NULL,
 			      NULL,
-                              panel_applet_marshal_VOID__ENUM_POINTER_OBJECT,
+                              panel_applet_marshal_VOID__ENUM_BOXED_OBJECT,
                               G_TYPE_NONE,
 			      3,
 			      PANEL_TYPE_PANEL_APPLET_BACKGROUND_TYPE,
-			      G_TYPE_POINTER,
+			      GDK_TYPE_COLOR,
 			      GDK_TYPE_PIXMAP);
 
 	panel_applet_signals [MOVE_FOCUS_OUT_OF_APPLET] =
@@ -1459,9 +1575,6 @@ panel_applet_setup (PanelApplet *applet)
 
 	g_signal_connect (applet, "popup_menu",
 			  G_CALLBACK (panel_applet_popup_menu), NULL);
-
-	priv->focusable_child = -1;
-	priv->hierarchy_changed_id  = 0;
 }
 
 GtkWidget *
